// Inspired by the DuckDuckGo Privacy Extension (https://github.com/duckduckgo/duckduckgo-privacy-extension)
// Original source for this file can be found here:
// https://github.com/duckduckgo/duckduckgo-privacy-extension/tree/develop/shared/js/background

/*
 * Copyright (C) 2012, 2016 DuckDuckGo, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/* For ESLint: List any global identifiers used in this file below */
/* global require log module chrome blockCounts exports */

'use strict';

const DDGsettings = require('./settings.es6');
const Tab = require('./classes/tab.es6');
const browserWrapper = require('./chrome-wrapper.es6');

class TabManager {
  constructor() {
    this.tabContainer = {};
  }

  /* This overwrites the current tab data for a given
     * id and is only called in three cases:
     * 1. When we rebuild saved tabs when the browser is restarted
     * 2. When a new tab is opened. See onUpdated listener below
     * 3. When we get a new main_frame request
     */
  create(tabData) {
    const normalizedData = browserWrapper.normalizeTabData(tabData);
    const newTab = new Tab(normalizedData);
    this.tabContainer[newTab.id] = newTab;
    return newTab;
  }

  delete(id) {
    delete this.tabContainer[id];
  }

  /* Called using either a chrome tab object or by id
     * get({tabId: ###});
     */
  get(tabData) {
    return this.tabContainer[tabData.tabId];
  }

  /* This will whitelist any open tabs with the same domain
     * list: name of the whitelist to update
     * domain: domain to whitelist
     * value: whitelist value, true or false
     */
  whitelistDomain(data) {
    this.setGlobalWhitelist(data.list, data.domain, data.value);

    for (const tabId in this.tabContainer) {
      const tab = this.tabContainer[tabId];
      if (tab.site && tab.site.domain === data.domain) {
        tab.site.setWhitelisted(data.list, data.value);
      }
    }
  }

  /* Update the whitelists kept in settings
     */
  static setGlobalWhitelist(list, domain, value) {
    const globalwhitelist = DDGsettings.getSetting(list) || {};

    if (value) {
      globalwhitelist[domain] = true;
    } else {
      delete globalwhitelist[domain];
    }

    DDGsettings.updateSetting(list, globalwhitelist);
  }

  /* This handles the new tab case. You have clicked to
     * open a new tab and haven't typed in a url yet.
     * This will fire an onUpdated event and we can create
     * an intital tab instance here. We'll update this instance
     * later on when webrequests start coming in.
     */
  static createOrUpdateTab(id, info) {
    /* eslint-disable-next-line no-use-before-define */
    if (!tabManager.get({ tabId: id })) {
      /* eslint-disable-next-line no-param-reassign */
      info.id = id;
      /* eslint-disable-next-line no-use-before-define */
      tabManager.create(info);
    } else {
      /* eslint-disable-next-line no-use-before-define */
      const tab = tabManager.get({ tabId: id });
      if (tab && info.status) {
        tab.status = info.status;
      }
    }
  }

  static updateTabUrl(request) {
    // Update tab data. This makes
    // sure we have the correct url after any https rewrites
    /* eslint-disable-next-line no-use-before-define */
    const tab = tabManager.get({ tabId: request.tabId });

    if (tab) {
      tab.statusCode = request.statusCode;
      if (tab.statusCode === 200) {
        tab.updateSite(request.url);
      }
    }
  }
}

const tabManager = new TabManager();


module.exports = {
  tabManager,
  TabManager,
};